@@ -11,41 +11,12 @@ from pathlib import Path
 from typing import Any, Dict, List, Tuple
 
 from blight import util
-from blight.enums import CodeModel, CompilerStage, Lang, OptLevel, Std
+from blight.enums import BlightTool, BuildTool, CodeModel, CompilerStage, Lang, OptLevel, Std
 from blight.exceptions import BlightError, BuildError, SkipRun
 from blight.protocols import CanonicalizedArgsProtocol, IndexedUndefinesProtocol, LangProtocol
 
 logger = logging.getLogger(__name__)
 
-BLIGHT_TOOL_MAP = {
-    "blight-cc": "CC",
-    "blight-c++": "CXX",
-    "blight-cpp": "CPP",
-    "blight-ld": "LD",
-    "blight-as": "AS",
-    "blight-ar": "AR",
-    "blight-strip": "AR",
-}
-
-TOOL_ENV_MAP = {
-    "CC": "cc",
-    "CXX": "c++",
-    "CPP": "cpp",
-    "LD": "ld",
-    "AS": "as",
-    "AR": "ar",
-    "STRIP": "strip",
-}
-
-TOOL_ENV_WRAPPER_MAP = {
-    "CC": "BLIGHT_WRAPPED_CC",
-    "CXX": "BLIGHT_WRAPPED_CXX",
-    "CPP": "BLIGHT_WRAPPED_CPP",
-    "LD": "BLIGHT_WRAPPED_LD",
-    "AS": "BLIGHT_WRAPPED_AS",
-    "AR": "BLIGHT_WRAPPED_AR",
-    "STRIP": "BLIGHT_WRAPPED_STRIP",
-}
 
 RESPONSE_FILE_RECURSION_LIMIT = 64
 """
@@ -83,17 +54,31 @@ class Tool:
     `Tool` instances cannot be created directory; a specific subclass must be used.
     """
 
+    @classmethod
+    def build_tool(cls) -> BuildTool:
+        """
+        Returns the `BuildTool` enum associated with this `Tool`.
+        """
+        return BuildTool(cls.__name__)
+
+    @classmethod
+    def blight_tool(cls) -> BlightTool:
+        """
+        Returns the `BlightTool` enum associated with this `Tool`.
+        """
+        return cls.build_tool().blight_tool
+
     @classmethod
     def wrapped_tool(cls) -> str:
         """
         Returns the executable name or path of the tool that this blight tool wraps.
         """
-        wrapped_tool = os.getenv(TOOL_ENV_WRAPPER_MAP[cls.__name__])
+        wrapped_tool = os.getenv(cls.blight_tool().env)
         if wrapped_tool is None:
-            raise BlightError(f"No wrapped tool found for {TOOL_ENV_MAP[cls.__name__]}")
+            raise BlightError(f"No wrapped tool found for {cls.build_tool()}")
         return wrapped_tool
 
-    def __init__(self, args):
+    def __init__(self, args: List[str]) -> None:
         if self.__class__ == Tool:
             raise NotImplementedError(f"can't instantiate {self.__class__.__name__} directly")
         self._args = args
@@ -103,7 +88,7 @@ class Tool:
         self._actions = util.load_actions()
         self._skip_run = False
 
-    def _fixup_env(self):
+    def _fixup_env(self) -> Dict[str, str]:
         """
         Fixes up `os.environ` to remove any references to blight's swizzled paths,
         if any are present.
@@ -112,18 +97,18 @@ class Tool:
         env["PATH"] = util.unswizzled_path()
         return env
 
-    def _before_run(self):
+    def _before_run(self) -> None:
         for action in self._actions:
             try:
                 action._before_run(self)
             except SkipRun:
                 self._skip_run = True
 
-    def _after_run(self):
+    def _after_run(self) -> None:
         for action in self._actions:
             action._after_run(self, run_skipped=self._skip_run)
 
-    def run(self):
+    def run(self) -> None:
         """
         Runs the wrapped tool with the original arguments.
         """
@@ -154,11 +139,11 @@ class Tool:
         }
 
     @property
-    def args(self):
+    def args(self) -> List[str]:
         return self._args
 
     @args.setter
-    def args(self, args_):
+    def args(self, args_: List[str]) -> None:
         self._args = args_
 
         # NOTE(ww): Modifying the effective arguments also propagates
@@ -168,7 +153,7 @@ class Tool:
         self._canonicalized_args = args_.copy()
 
     @property
-    def canonicalized_args(self):
+    def canonicalized_args(self) -> List[str]:
         # NOTE(ww): `canonicalized_args` doesn't need an explicit setter property,
         # since all specializations of it are expected to modify the underlying
         # list.
@@ -479,13 +464,13 @@ class ResponseFileMixin:
     the name of this mixin.
     """
 
-    def _expand_response_file(self, response_file, working_dir, level):
+    def _expand_response_file(
+        self, response_file: Path, working_dir: Path, level: int
+    ) -> List[str]:
         if level >= RESPONSE_FILE_RECURSION_LIMIT:
             logger.debug(f"recursion limit exceeded: {response_file} in {working_dir}")
             return []
 
-        response_file = Path(response_file[1:])
-
         # Non-absolute response files are resolved relative to `working_dir`, which
         # begins at the CWD initially and changes to the parent directory of the
         # including file for nested response files.
@@ -503,27 +488,36 @@ class ResponseFileMixin:
             args = util.insert_items_at_idx(
                 args,
                 idx,
-                self._expand_response_file(nested_rf, response_file.parent.resolve(), level + 1),
+                self._expand_response_file(
+                    Path(nested_rf[1:]), response_file.parent.resolve(), level + 1
+                ),
             )
 
         return args
 
     @property
-    def canonicalized_args(self):
+    def canonicalized_args(self) -> List[str]:
         """
         Overrides the behavior of `Tool.canonicalized_args`, expanding any response file arguments
         in a depth-first manner.
         """
 
+        # NOTE(ww): This method badly needs some typechecking TLC.
+        # The `super()` call to `canonicalized_args` probably needs to be handled
+        # with a `self: CanonicalizedArgsProtocol` hint, but that causes other problems
+        # related to mypy's ability to see `_expand_response_file`.
+
         response_files = [
             (idx, arg)
-            for (idx, arg) in enumerate(super().canonicalized_args)
+            for (idx, arg) in enumerate(super().canonicalized_args)  # type: ignore
             if arg.startswith("@")
         ]
-        expanded_args = super().canonicalized_args
+        expanded_args = super().canonicalized_args  # type: ignore
         for idx, response_file in response_files:
             expanded_args = util.insert_items_at_idx(
-                expanded_args, idx, self._expand_response_file(response_file, self.cwd, 0)
+                expanded_args,
+                idx,
+                self._expand_response_file(Path(response_file[1:]), self.cwd, 0),  # type: ignore
             )
 
         self._canonicalized_args = expanded_args
@@ -643,7 +637,7 @@ class CompilerTool(ResponseFileMixin, Tool, StdMixin, OptMixin, DefinesMixin, Co
     Like `Tool`, `CompilerTool` cannot be instantiated directly.
     """
 
-    def __init__(self, args):
+    def __init__(self, args) -> None:
         if self.__class__ == CompilerTool:
             raise NotImplementedError(f"can't instantiate {self.__class__.__name__} directly")
         super().__init__(args)
